/*
************************************************************************
*
*   %M% - ShowProp* commands
*
*   Copyright (c) 1994-96
*
*   ETH Zuerich
*   Institut fuer Molekularbiologie und Biophysik
*   ETH-Hoenggerberg
*   CH-8093 Zuerich
*
*   SPECTROSPIN AG
*   Industriestr. 26
*   CH-8117 Faellanden
*
*   All Rights Reserved
*
*   Date of last modification : %E%
*   Pathname of SCCS file     : %P%
*   SCCS identification       : %I%
*
************************************************************************
*/

#include <cmd_prop.h>

#include <stdio.h>
#include <string.h>

#include <pu.h>
#include <prop_tab.h>
#include <data_hand.h>
#include <prim_hand.h>

typedef struct {
  int entNo;
  AttrP attrP;
} PropDescr;

static void
getProp(AttrP attrP, PropDescr *descrP)
{
  if (descrP->entNo == 0)
    descrP->attrP = attrP;
  else if (descrP->attrP != attrP)
    descrP->attrP = NULL;

  descrP->entNo++;
}

static void
getPropAtom(DhAtomP atomP, void *clientData)
{
  getProp(DhAtomGetProp(atomP), clientData);
}

static void
getPropBond(DhBondP bondP, DhAtomP atom1P, DhAtomP atom2P, void *clientData)
{
  getProp(DhBondGetProp(bondP), clientData);
}

static void
getPropDist(DhDistP distP, DhAtomP atom1P, DhAtomP atom2P, void *clientData)
{
  getProp(DhDistGetProp(distP), clientData);
}

static void
getPropPrim(PrimObjP primP, void *clientData)
{
  getProp(PrimGetProp(primP), clientData);
}

ErrCode
ExAttr(char *cmd)
{
  AttrDescr descr;
  PropRefP refP;
  DataEntityType entType;
  AttrP attrP;
  PuTextWindow textW;
  char *name, buf[100], *str;
  DSTR macro;

  descr.entNo = 0;
  descr.attrP = NULL;

  refP = PropGetRef(PROP_SELECTED, FALSE);

  if (strcmp(cmd, "AttrAtom") == 0) {
    DhApplyAtom(refP, getAttrAtom, &descr);
    entType = DE_ATOM;
  } else if (strcmp(cmd, "AttrBond") == 0) {
    DhApplyBond(refP, getAttrBond, &descr);
    entType = DE_BOND;
  } else if (strcmp(cmd, "AttrDist") == 0) {
    DhApplyDist(refP, getAttrDist, &descr);
    entType = DE_DIST;
  } else {
    PrimApply(PT_ALL, refP, getAttrPrim, &descr);
    entType = DE_PRIM;
  }

  if (descr.entNo == 0) {
    CipSetError("no item selected");
    return EC_ERROR;
  }

  attrP = descr.attrP;
  if (attrP == NULL) {
    CipSetError("not all selected items have same attributes");
    return EC_ERROR;
  }

  textW = PuCreateTextWindow(cmd);
  macro = DStrNew();
  name = cmd + 4;

  (void) sprintf(buf, "Values of attr %d:\n\n", AttrGetIndex(attrP));
  PuWriteStr(textW, buf);

  (void) sprintf(buf, "Color:           %5.3f %5.3f %5.3f\n",
      attrP->colR, attrP->colG, attrP->colB);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Color%s %5.3f %5.3f %5.3f\n",
      name, attrP->colR, attrP->colG, attrP->colB);
  DStrAppStr(macro, buf);
  if (entType == DE_DIST || entType == DE_PRIM) {
    (void) sprintf(buf, "Secondary Color: %5.3f %5.3f %5.3f\n",
	attrP->secColR, attrP->secColG, attrP->secColB);
    PuWriteStr(textW, buf);
    (void) sprintf(buf, "Tint%s %5.3f %5.3f %5.3f\n",
	name, attrP->secColR, attrP->secColG, attrP->secColB);
    DStrAppStr(macro, buf);
  }

  (void) sprintf(buf, "Ambient Coeff.:  %5.3f\n", attrP->ambCoeff);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Diffuse Coeff.:  %5.3f\n", attrP->diffCoeff);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Specular Coeff.: %5.3f\n", attrP->specCoeff);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Shininess:       %.1f\n", attrP->shininess);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Reflection:      %5.3f\n", attrP->reflect);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Opacity:         %5.3f\n", attrP->alpha);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Refraction:      %5.3f\n", attrP->refract);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Material%s %5.3f %5.3f %5.3f %.1f %5.3f %5.3f %5.3f\n",
      name, attrP->ambCoeff, attrP->diffCoeff, attrP->specCoeff,
      attrP->shininess, attrP->reflect, attrP->alpha, attrP->refract);
  DStrAppStr(macro, buf);

  (void) sprintf(buf, "Texture Name:    %s\n", attrP->texture);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Texture Scale:   %.2f\n", attrP->textureScale);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Bumpiness:       %5.3f\n", attrP->bumpDepth);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Bump Scale:      %.2f\n", attrP->bumpScale);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Texture%s '%s' %.2f %5.3f %.2f\n",
      name, attrP->texture, attrP->textureScale,
      attrP->bumpDepth, attrP->bumpScale);
  DStrAppStr(macro, buf);

  if (entType == DE_ATOM) {
    PuWriteStr(textW, "Style:           ");
    if (attrP->atomStyle == AS_INVISIBLE)
      str = "invisible";
    else if (attrP->atomStyle == AS_SPHERE)
      str = "sphere";
    else
      str = "tetrahedron";
    (void) sprintf(buf, "Style:           %s\n", str);
    PuWriteStr(textW, buf);
    (void) sprintf(buf, "StyleAtom %s\n", str);
    DStrAppStr(macro, buf);

    (void) sprintf(buf, "Label:           %s\n", attrP->labelFormat);
    PuWriteStr(textW, buf);
    (void) sprintf(buf, "LabelAtom '%s'\n", attrP->labelFormat);
    DStrAppStr(macro, buf);
  } else if (entType == DE_BOND) {
    if (attrP->bondStyle == BS_INVISIBLE)
      str = "invisible";
    else if (attrP->bondStyle == BS_LINE)
      str = "line";
    else if (attrP->bondStyle == BS_HALF_LINE)
      str = "half_line";
    else if (attrP->bondStyle == BS_CYLINDER)
      str = "cylinder";
    else if (attrP->bondStyle == BS_HALF_CYLINDER)
      str = "half_cylinder";
    else if (attrP->bondStyle == BS_NEON)
      str = "neon";
    else
      str = "half_neon";
    (void) sprintf(buf, "Style:           %s\n", str);
    PuWriteStr(textW, buf);
    (void) sprintf(buf, "StyleBond %s\n", str);
    DStrAppStr(macro, buf);
  } else if (entType == DE_DIST) {
    if (attrP->distStyle == DS_INVISIBLE)
      str = "invisible";
    else if (attrP->distStyle == DS_LINE)
      str = "line";
    else if (attrP->distStyle == DS_VIOL)
      str = "viol";
    else if (attrP->distStyle == DS_CYLINDER)
      str = "cylinder";
    else
      str = "neon";
    (void) sprintf(buf, "Style:           %s\n", str);
    PuWriteStr(textW, buf);
    (void) sprintf(buf, "StyleDist %s\n", str);
    DStrAppStr(macro, buf);

    if (attrP->distLabel == DL_INVISIBLE)
      str = "invisible";
    else if (attrP->distLabel == DL_VAL)
      str = "val";
    else if (attrP->distLabel == DL_LIMIT)
      str = "limit";
    else
      str = "viol";
    (void) sprintf(buf, "Label:           %s\n", str);
    PuWriteStr(textW, buf);
    (void) sprintf(buf, "StyleDist %s\n", str);
    DStrAppStr(macro, buf);
  }

  (void) sprintf(buf, "Radius:          %.3f\n", attrP->radius);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Radius%s %.3f\n", name, attrP->radius);
  DStrAppStr(macro, buf);

  (void) sprintf(buf, "Line Style:      %s\n", str);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Line Width:      %.1f\n", attrP->lineWidth);
  PuWriteStr(textW, buf);
  if (attrP->lineStyle == SG_LINE_SOLID)
    str = "solid";
  else
    str = "dashed";
  (void) sprintf(buf, "Line%s %s %.1f\n", name, str, attrP->lineWidth);
  DStrAppStr(macro, buf);

  if (attrP->shadeModel == -1)
    str = "none";
  else if (attrP->shadeModel == SG_SHADE_FLAT)
    str = "flat";
  else if (attrP->shadeModel == SG_SHADE_GOURAUD)
    str = "gouraud";
  else if (attrP->shadeModel == SG_SHADE_PHONG)
    str = "phong";
  else if (attrP->shadeModel == SG_SHADE_DOTS)
    str = "dots";
  else
    str = "lines";
  (void) sprintf(buf, "Shading:         %s\n", str);
  PuWriteStr(textW, buf);
  (void) sprintf(buf, "Shade%s %s\n", name, str);
  DStrAppStr(macro, buf);

  if (entType == DE_PRIM) {
    (void) sprintf(buf, "Font Size:       %5.3f\n", attrP->fontSize);
    PuWriteStr(textW, buf);
    (void) sprintf(buf, "SizeText %5.3f\n", attrP->fontSize);
    DStrAppStr(macro, buf);
  }

  PuWriteStr(textW, "\nMacro:\n\n");
  PuWriteStr(textW, DStrToStr(macro));

  DStrFree(macro);

  return EC_OK;
}
